import {
  ButtonInteraction,
  CacheType,
  ChatInputCommandInteraction,
  Interaction,
  ModalSubmitInteraction,
} from "discord.js";
import EventEmitter from "events";
import DiscordService from "Services/discord/discord.service";

export default class DiscordController extends EventEmitter {
  private discordService!: DiscordService;

  constructor() {
    super();
    this.discordService = new DiscordService();
    // log when running
    this.discordService.client.once("ready", () => {
      console.log("Listening...");
    });
    // listen for interactions
    this.discordService.client.on(
      "interactionCreate",
      this.handleInteraction.bind(this)
    );
  }

  async init() {
    await this.discordService.init();
  }

  async handleInteraction(interaction: Interaction<CacheType>) {
    if (interaction.isModalSubmit()) {
      await this.handleModalSubmit(interaction);
    } else if (interaction.isChatInputCommand()) {
      await this.handleChatInputCommand(interaction);
    } else if (interaction.isButton()) {
      await this.handleButton(interaction);
    }
  }

  async handleButton(interaction: ButtonInteraction<CacheType>) {
    const { customId } = interaction;

    if (customId === "quoteModal") {
      this.discordService.handleQuoteModal(interaction);
    }
  }

  async handleChatInputCommand(
    interaction: ChatInputCommandInteraction<CacheType>
  ) {
    const { commandName } = interaction;

    switch (commandName) {
      case "quote":
        await this.discordService.handleQuote(interaction);
        return;
      case "quotemodal":
        await this.discordService.handleQuoteModal(interaction);
        return;
      case "actions":
        await this.discordService.handleActions(interaction);
        return;
      default:
        break;
    }
  }

  async handleModalSubmit(interaction: ModalSubmitInteraction<CacheType>) {
    const { customId } = interaction;

    switch (customId) {
      case "quoteModal":
        await this.discordService.handleQuote(interaction);
        return;
      default:
        break;
    }
  }
}
